@@ -1,4 +1,4 @@
-# Copyright 2006-2016 Roger Bivand
+# Copyright 2006-2021 Roger Bivand
 
 readOGR <- function(dsn, layer, verbose=TRUE, p4s=NULL, 
         stringsAsFactors=as.logical(NA),
@@ -7,7 +7,9 @@ readOGR <- function(dsn, layer, verbose=TRUE, p4s=NULL,
         disambiguateFIDs=FALSE, addCommentsToPolygons=TRUE, encoding=NULL,
         use_iconv=FALSE, swapAxisOrder=FALSE, require_geomType=NULL,
         integer64="no.loss", GDAL1_integer64_policy=FALSE,
-        morphFromESRI=NULL, dumpSRS=FALSE, enforce_xy=NULL) {
+        morphFromESRI=NULL, dumpSRS=FALSE, enforce_xy=NULL,
+        D3_if_2D3D_points=FALSE, missing_3D=0) {
+  .Deprecated("", package="rgdal", msg="OGR support is provided by the sf and terra packages among others")
 	if (missing(dsn)) stop("missing dsn")
         stopifnot(is.character(dsn))
         stopifnot(length(dsn) == 1L)
@@ -61,7 +63,8 @@ readOGR <- function(dsn, layer, verbose=TRUE, p4s=NULL,
 	suppressMessages(ogr_info <- ogrInfo(dsn=dsn, layer=layer,
             encoding=encoding, use_iconv=use_iconv,
             swapAxisOrder=swapAxisOrder, require_geomType=require_geomType,
-            morphFromESRI=morphFromESRI, dumpSRS=dumpSRS))
+            morphFromESRI=morphFromESRI, dumpSRS=dumpSRS,
+            D3_if_2D3D_points=D3_if_2D3D_points))
         HAS_FEATURES <- TRUE
         if (!ogr_info$have_features) {
             if (dropNULLGeometries) {
@@ -262,11 +265,22 @@ readOGR <- function(dsn, layer, verbose=TRUE, p4s=NULL,
             eType <- eType[!isNULL]
             with_z <- with_z[!isNULL]
         }   
-     
+        elevated <- attr(ogr_info$with_z, "elevated")
 	u_with_z <- unique(sort(with_z))
-	if (length(u_with_z) != 1L) stop(
-		paste("Multiple # dimensions:", 
-			paste((u_with_z + 2), collapse=":")))
+	if (length(u_with_z) != 1L) {
+          if (elevated) {
+            if (pointDropZ) {
+              warning("Dropping mixed third dimension")
+              u_with_z <- 0L
+            } else {
+              warning("Setting any missing third dimension to ", missing_3D)
+              u_with_z <- 1L
+            }
+          } else {
+            stop(paste("Multiple # dimensions:", paste((u_with_z + 2), 
+              collapse=":")))
+          }
+        }
 	if (u_with_z < 0 || u_with_z > 1) stop(
 		paste("Invalid # dimensions:", (u_with_z + 2)))
 
@@ -333,15 +347,27 @@ readOGR <- function(dsn, layer, verbose=TRUE, p4s=NULL,
 				function(x) c(x[[1]][[1]], x[[1]][[2]])))
                     }
 		} else {
+                  if (elevated) {
                     if (swapAxisOrder) {
 			coords <- do.call("rbind", lapply(gFeatures, 
-				function(x) c(x[[1]][[2]], x[[1]][[1]],
-				x[[1]][[3]])))
+			    function(x) c(x[[1]][[2]], x[[1]][[1]], 
+                            ifelse(length(x[[1]]) == 2L, missing_3D, x[[1]][[3]]))))
                     } else {
 			coords <- do.call("rbind", lapply(gFeatures, 
-				function(x) c(x[[1]][[1]], x[[1]][[2]],
-				x[[1]][[3]])))
-                    }  
+			    function(x) c(x[[1]][[1]], x[[1]][[2]],
+                            ifelse(length(x[[1]]) == 2L, missing_3D, x[[1]][[3]]))))
+                    } 
+                  } else {
+                    if (swapAxisOrder) {
+			coords <- do.call("rbind", lapply(gFeatures, 
+			    function(x) c(x[[1]][[2]], x[[1]][[1]],
+                            ifelse(length(x[[1]]) == 2L, missing_3D, x[[1]][[3]]))))
+                    } else {
+			coords <- do.call("rbind", lapply(gFeatures, 
+			    function(x) c(x[[1]][[1]], x[[1]][[2]], 
+                            ifelse(length(x[[1]]) == 2L, missing_3D, x[[1]][[3]]))))
+                    } 
+                  } 
 		}
 #		data <- data.frame(dlist)
 		row.names(data) <- NULL
@@ -448,6 +474,7 @@ readOGR <- function(dsn, layer, verbose=TRUE, p4s=NULL,
 }
 
 showWKT <- function(p4s, file=NULL, morphToESRI=FALSE, enforce_xy=NULL) {
+  .Deprecated("", package="rgdal", msg="OGR support is provided by the sf and terra packages among others")
 
 	if (!is.character(p4s)) stop("invalid p4s object")
         stopifnot(length(p4s) == 1)
@@ -490,6 +517,7 @@ showP4 <- function(wkt, morphFromESRI=FALSE, enforce_xy=NULL) {
 
 
 showEPSG <- function(p4s, enforce_xy=NULL) {
+  .Deprecated("", package="rgdal", msg="OGR support is provided by the sf and terra packages among others")
 
 	if (!is.character(p4s)) stop("invalid p4s object")
         stopifnot(length(p4s) == 1L)
@@ -587,6 +615,7 @@ showSRID <- function(inSRID, format="WKT2", multiline="NO", enforce_xy=NULL, EPS
     if (substring(inSRID, 1, 2) == "BO") in_format = 3L
     if (substring(inSRID, 1, 1) == "S") in_format = 3L
     if (substring(inSRID, 1, 2) == "CO") in_format = 3L
+    if (substring(inSRID, 1, 3) == "ENG") in_format = 3L
     if (substring(inSRID, 1, 4) == "EPSG") in_format = 4L
     if (substring(inSRID, 1, 4) == "ESRI") in_format = 5L
     if (substring(inSRID, 1, 3) == "OGC") in_format = 5L
@@ -598,7 +627,7 @@ showSRID <- function(inSRID, format="WKT2", multiline="NO", enforce_xy=NULL, EPS
     } else {
         prefer_proj <- get_prefer_proj()
     }
-    if (in_format == 4L) {
+    if (!is.na(in_format) && in_format == 4L) {
         if (EPSG_to_init && !prefer_proj) {
             in_format = 1L
             inSRID <- paste0("+init=epsg:", substring(inSRID, 6, nchar(inSRID)))
@@ -643,7 +672,7 @@ showSRID <- function(inSRID, format="WKT2", multiline="NO", enforce_xy=NULL, EPS
                 (length(grep("ellps|ELLIPSOID", c(res))) == 0L)) {
                 if (length(grep("datum|DATUM", c(res))) == 0L) {
                     msg <- paste0("Discarded ellps ", attr(res, "ellps"),
-                        " in CRS definition: ", c(res))
+                        " in Proj4 definition: ", c(res))
                 } else {
                     msg <- ""
                 }
@@ -662,12 +691,13 @@ showSRID <- function(inSRID, format="WKT2", multiline="NO", enforce_xy=NULL, EPS
                 }
             }
 #warning("Discarded ellps ", attr(res, "ellps"),
-#                    " in CRS definition")
+#                    " in Proj4 definition")
             if ((!is.null(attr(res, "datum"))) 
                 && (nchar(attr(res, "datum")) > 0L)
-                && (length(grep("datum|DATUM", c(res))) == 0L)) {
+# spatsoc tripped PROJ 8.0.0 ENSEMBLE 2021-02-22
+                && (length(grep("datum|DATUM|ENSEMBLE", c(res))) == 0L)) {
                 msg <- paste0("Discarded datum ", attr(res, "datum"),
-                    " in CRS definition")
+                    " in Proj4 definition")
                 if (!no_towgs84 && (length(grep("towgs84", c(res))) > 0L))
                     msg <- paste0(msg, ",\n but +towgs84= values preserved")
                 if (get_P6_datum_hard_fail()) stop(msg)
